@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require 'test/unit'
 require 'open-uri'
 require 'webrick'
@@ -11,28 +12,39 @@ class TestOpenURI < Test::Unit::TestCase
 
   NullLog = Object.new
   def NullLog.<<(arg)
+    #puts arg if / INFO / !~ arg
   end
 
-  def with_http
+  def with_http(log_tester=lambda {|log| assert_equal([], log) })
+    log = []
+    logger = WEBrick::Log.new(log, WEBrick::BasicLog::WARN)
     Dir.mktmpdir {|dr|
       srv = WEBrick::HTTPServer.new({
         :DocumentRoot => dr,
         :ServerType => Thread,
-        :Logger => WEBrick::Log.new(NullLog),
+        :Logger => logger,
         :AccessLog => [[NullLog, ""]],
         :BindAddress => '127.0.0.1',
         :Port => 0})
       _, port, _, host = srv.listeners[0].addr
-      begin
-        srv.start
-        yield srv, dr, "http://#{host}:#{port}"
-      ensure
-        srv.shutdown
-        until srv.status == :Stop
-          sleep 0.1
+      server_thread = srv.start
+      server_thread2 = Thread.new {
+        server_thread.join
+        if log_tester
+          log_tester.call(log)
         end
-      end
+      }
+      client_thread = Thread.new {
+        begin
+          yield srv, dr, "http://#{host}:#{port}", server_thread, log
+        ensure
+          srv.shutdown
+        end
+      }
+      assert_join_threads([client_thread, server_thread2])
     }
+  ensure
+    WEBrick::Utils::TimeoutHandler.terminate
   end
 
   def with_env(h)
@@ -56,10 +68,32 @@ class TestOpenURI < Test::Unit::TestCase
     @proxies.each_with_index {|k, i| ENV[k] = @old_proxies[i] }
   end
 
+  def test_deprecated_kernel_open
+    with_http {|srv, dr, url|
+      srv.mount_proc("/foo200", lambda { |req, res| res.body = "foo200" } )
+      assert_warning(/calling URI.open via Kernel#open is deprecated, call URI.open directly/) {
+        open("#{url}/foo200") {|f|
+          assert_equal("200", f.status[0])
+          assert_equal("foo200", f.read)
+        }
+      }
+    }
+  end
+
+  def test_200_uri_open
+    with_http {|srv, dr, url|
+      srv.mount_proc("/urifoo200", lambda { |req, res| res.body = "urifoo200" } )
+      URI.open("#{url}/urifoo200") {|f|
+        assert_equal("200", f.status[0])
+        assert_equal("urifoo200", f.read)
+      }
+    }
+  end
+
   def test_200
     with_http {|srv, dr, url|
       srv.mount_proc("/foo200", lambda { |req, res| res.body = "foo200" } )
-      open("#{url}/foo200") {|f|
+      URI.open("#{url}/foo200") {|f|
         assert_equal("200", f.status[0])
         assert_equal("foo200", f.read)
       }
@@ -70,7 +104,7 @@ class TestOpenURI < Test::Unit::TestCase
     with_http {|srv, dr, url|
       content = "foo200big"*10240
       srv.mount_proc("/foo200big", lambda { |req, res| res.body = content } )
-      open("#{url}/foo200big") {|f|
+      URI.open("#{url}/foo200big") {|f|
         assert_equal("200", f.status[0])
         assert_equal(content, f.read)
       }
@@ -78,8 +112,12 @@ class TestOpenURI < Test::Unit::TestCase
   end
 
   def test_404
-    with_http {|srv, dr, url|
-      exc = assert_raise(OpenURI::HTTPError) { open("#{url}/not-exist") {} }
+    log_tester = lambda {|server_log|
+      assert_equal(1, server_log.length)
+      assert_match(%r{ERROR `/not-exist' not found}, server_log[0])
+    }
+    with_http(log_tester) {|srv, dr, url, server_thread, server_log|
+      exc = assert_raise(OpenURI::HTTPError) { URI.open("#{url}/not-exist") {} }
       assert_equal("404", exc.io.status[0])
     }
   end
@@ -88,7 +126,7 @@ class TestOpenURI < Test::Unit::TestCase
     with_http {|srv, dr, url|
       srv.mount_proc("/foo_ou", lambda { |req, res| res.body = "foo_ou" } )
       u = URI("#{url}/foo_ou")
-      open(u) {|f|
+      URI.open(u) {|f|
         assert_equal("200", f.status[0])
         assert_equal("foo_ou", f.read)
       }
@@ -96,7 +134,7 @@ class TestOpenURI < Test::Unit::TestCase
   end
 
   def test_open_too_many_arg
-    assert_raise(ArgumentError) { open("http://192.0.2.1/tma", "r", 0666, :extra) {} }
+    assert_raise(ArgumentError) { URI.open("http://192.0.2.1/tma", "r", 0666, :extra) {} }
   end
 
   def test_read_timeout
@@ -124,29 +162,47 @@ class TestOpenURI < Test::Unit::TestCase
     }
   end
 
+  def test_open_timeout
+    assert_raise(Net::OpenTimeout) do
+      URI("http://example.com/").read(open_timeout: 0.000001)
+    end if false # avoid external resources in tests
+
+    with_http {|srv, dr, url|
+      url += '/'
+      srv.mount_proc('/', lambda { |_, res| res.body = 'hi' })
+      begin
+        URI(url).read(open_timeout: 0.000001)
+      rescue Net::OpenTimeout
+        # not guaranteed to fire, since the kernel negotiates the
+        # TCP connection even if the server thread is sleeping
+      end
+      assert_equal 'hi', URI(url).read(open_timeout: 60), 'should not timeout'
+    }
+  end
+
   def test_invalid_option
-    assert_raise(ArgumentError) { open("http://127.0.0.1/", :invalid_option=>true) {} }
+    assert_raise(ArgumentError) { URI.open("http://127.0.0.1/", :invalid_option=>true) {} }
   end
 
   def test_mode
     with_http {|srv, dr, url|
       srv.mount_proc("/mode", lambda { |req, res| res.body = "mode" } )
-      open("#{url}/mode", "r") {|f|
+      URI.open("#{url}/mode", "r") {|f|
         assert_equal("200", f.status[0])
         assert_equal("mode", f.read)
       }
-      open("#{url}/mode", "r", 0600) {|f|
+      URI.open("#{url}/mode", "r", 0600) {|f|
         assert_equal("200", f.status[0])
         assert_equal("mode", f.read)
       }
-      assert_raise(ArgumentError) { open("#{url}/mode", "a") {} }
-      open("#{url}/mode", "r:us-ascii") {|f|
+      assert_raise(ArgumentError) { URI.open("#{url}/mode", "a") {} }
+      URI.open("#{url}/mode", "r:us-ascii") {|f|
         assert_equal(Encoding::US_ASCII, f.read.encoding)
       }
-      open("#{url}/mode", "r:utf-8") {|f|
+      URI.open("#{url}/mode", "r:utf-8") {|f|
         assert_equal(Encoding::UTF_8, f.read.encoding)
       }
-      assert_raise(ArgumentError) { open("#{url}/mode", "r:invalid-encoding") {} }
+      assert_raise(ArgumentError) { URI.open("#{url}/mode", "r:invalid-encoding") {} }
     }
   end
 
@@ -154,7 +210,7 @@ class TestOpenURI < Test::Unit::TestCase
     with_http {|srv, dr, url|
       srv.mount_proc("/without_block", lambda { |req, res| res.body = "without_block" } )
       begin
-        f = open("#{url}/without_block")
+        f = URI.open("#{url}/without_block")
         assert_equal("200", f.status[0])
         assert_equal("without_block", f.read)
       ensure
@@ -163,12 +219,35 @@ class TestOpenURI < Test::Unit::TestCase
     }
   end
 
+  def test_close_in_block_small
+    with_http {|srv, dr, url|
+      srv.mount_proc("/close200", lambda { |req, res| res.body = "close200" } )
+      assert_nothing_raised {
+        URI.open("#{url}/close200") {|f|
+          f.close
+        }
+      }
+    }
+  end
+
+  def test_close_in_block_big
+    with_http {|srv, dr, url|
+      content = "close200big"*10240
+      srv.mount_proc("/close200big", lambda { |req, res| res.body = content } )
+      assert_nothing_raised {
+        URI.open("#{url}/close200big") {|f|
+          f.close
+        }
+      }
+    }
+  end
+
   def test_header
     myheader1 = 'barrrr'
     myheader2 = nil
     with_http {|srv, dr, url|
       srv.mount_proc("/h/") {|req, res| myheader2 = req['myheader']; res.body = "foo" }
-      open("#{url}/h/", 'MyHeader'=>myheader1) {|f|
+      URI.open("#{url}/h/", 'MyHeader'=>myheader1) {|f|
         assert_equal("foo", f.read)
         assert_equal(myheader1, myheader2)
       }
@@ -177,75 +256,81 @@ class TestOpenURI < Test::Unit::TestCase
 
   def test_multi_proxy_opt
     assert_raise(ArgumentError) {
-      open("http://127.0.0.1/", :proxy_http_basic_authentication=>true, :proxy=>true) {}
+      URI.open("http://127.0.0.1/", :proxy_http_basic_authentication=>true, :proxy=>true) {}
     }
   end
 
   def test_non_http_proxy
     assert_raise(RuntimeError) {
-      open("http://127.0.0.1/", :proxy=>URI("ftp://127.0.0.1/")) {}
+      URI.open("http://127.0.0.1/", :proxy=>URI("ftp://127.0.0.1/")) {}
     }
   end
 
   def test_proxy
     with_http {|srv, dr, url|
-      log = ''
+      proxy_log = StringIO.new(''.dup)
+      proxy_logger = WEBrick::Log.new(proxy_log, WEBrick::BasicLog::WARN)
+      proxy_auth_log = ''.dup
       proxy = WEBrick::HTTPProxyServer.new({
         :ServerType => Thread,
-        :Logger => WEBrick::Log.new(NullLog),
+        :Logger => proxy_logger,
         :AccessLog => [[NullLog, ""]],
         :ProxyAuthProc => lambda {|req, res|
-          log << req.request_line
+          proxy_auth_log << req.request_line
         },
         :BindAddress => '127.0.0.1',
         :Port => 0})
       _, proxy_port, _, proxy_host = proxy.listeners[0].addr
       proxy_url = "http://#{proxy_host}:#{proxy_port}/"
       begin
-        proxy.start
+        proxy_thread = proxy.start
         srv.mount_proc("/proxy", lambda { |req, res| res.body = "proxy" } )
-        open("#{url}/proxy", :proxy=>proxy_url) {|f|
+        URI.open("#{url}/proxy", :proxy=>proxy_url) {|f|
           assert_equal("200", f.status[0])
           assert_equal("proxy", f.read)
         }
-        assert_match(/#{Regexp.quote url}/, log); log.clear
-        open("#{url}/proxy", :proxy=>URI(proxy_url)) {|f|
+        assert_match(/#{Regexp.quote url}/, proxy_auth_log); proxy_auth_log.clear
+        URI.open("#{url}/proxy", :proxy=>URI(proxy_url)) {|f|
           assert_equal("200", f.status[0])
           assert_equal("proxy", f.read)
         }
-        assert_match(/#{Regexp.quote url}/, log); log.clear
-        open("#{url}/proxy", :proxy=>nil) {|f|
+        assert_match(/#{Regexp.quote url}/, proxy_auth_log); proxy_auth_log.clear
+        URI.open("#{url}/proxy", :proxy=>nil) {|f|
           assert_equal("200", f.status[0])
           assert_equal("proxy", f.read)
         }
-        assert_equal("", log); log.clear
+        assert_equal("", proxy_auth_log); proxy_auth_log.clear
         assert_raise(ArgumentError) {
-          open("#{url}/proxy", :proxy=>:invalid) {}
+          URI.open("#{url}/proxy", :proxy=>:invalid) {}
         }
-        assert_equal("", log); log.clear
+        assert_equal("", proxy_auth_log); proxy_auth_log.clear
         with_env("http_proxy"=>proxy_url) {
           # should not use proxy for 127.0.0.0/8.
-          open("#{url}/proxy") {|f|
+          URI.open("#{url}/proxy") {|f|
             assert_equal("200", f.status[0])
             assert_equal("proxy", f.read)
           }
         }
-        assert_equal("", log); log.clear
+        assert_equal("", proxy_auth_log); proxy_auth_log.clear
       ensure
         proxy.shutdown
+        proxy_thread.join
       end
+      assert_equal("", proxy_log.string)
     }
   end
 
-  def test_proxy_http_basic_authentication
+  def test_proxy_http_basic_authentication_failure
     with_http {|srv, dr, url|
-      log = ''
+      proxy_log = StringIO.new(''.dup)
+      proxy_logger = WEBrick::Log.new(proxy_log, WEBrick::BasicLog::WARN)
+      proxy_auth_log = ''.dup
       proxy = WEBrick::HTTPProxyServer.new({
         :ServerType => Thread,
-        :Logger => WEBrick::Log.new(NullLog),
+        :Logger => proxy_logger,
         :AccessLog => [[NullLog, ""]],
         :ProxyAuthProc => lambda {|req, res|
-          log << req.request_line
+          proxy_auth_log << req.request_line
           if req["Proxy-Authorization"] != "Basic #{['user:pass'].pack('m').chomp}"
             raise WEBrick::HTTPStatus::ProxyAuthenticationRequired
           end
@@ -255,25 +340,93 @@ class TestOpenURI < Test::Unit::TestCase
       _, proxy_port, _, proxy_host = proxy.listeners[0].addr
       proxy_url = "http://#{proxy_host}:#{proxy_port}/"
       begin
-        proxy.start
+        th = proxy.start
         srv.mount_proc("/proxy", lambda { |req, res| res.body = "proxy" } )
-        exc = assert_raise(OpenURI::HTTPError) { open("#{url}/proxy", :proxy=>proxy_url) {} }
+        exc = assert_raise(OpenURI::HTTPError) { URI.open("#{url}/proxy", :proxy=>proxy_url) {} }
         assert_equal("407", exc.io.status[0])
-        assert_match(/#{Regexp.quote url}/, log); log.clear
-        open("#{url}/proxy",
+        assert_match(/#{Regexp.quote url}/, proxy_auth_log); proxy_auth_log.clear
+      ensure
+        proxy.shutdown
+        th.join
+      end
+      assert_match(/ERROR WEBrick::HTTPStatus::ProxyAuthenticationRequired/, proxy_log.string)
+    }
+  end
+
+  def test_proxy_http_basic_authentication_success
+    with_http {|srv, dr, url|
+      proxy_log = StringIO.new(''.dup)
+      proxy_logger = WEBrick::Log.new(proxy_log, WEBrick::BasicLog::WARN)
+      proxy_auth_log = ''.dup
+      proxy = WEBrick::HTTPProxyServer.new({
+        :ServerType => Thread,
+        :Logger => proxy_logger,
+        :AccessLog => [[NullLog, ""]],
+        :ProxyAuthProc => lambda {|req, res|
+          proxy_auth_log << req.request_line
+          if req["Proxy-Authorization"] != "Basic #{['user:pass'].pack('m').chomp}"
+            raise WEBrick::HTTPStatus::ProxyAuthenticationRequired
+          end
+        },
+        :BindAddress => '127.0.0.1',
+        :Port => 0})
+      _, proxy_port, _, proxy_host = proxy.listeners[0].addr
+      proxy_url = "http://#{proxy_host}:#{proxy_port}/"
+      begin
+        th = proxy.start
+        srv.mount_proc("/proxy", lambda { |req, res| res.body = "proxy" } )
+        URI.open("#{url}/proxy",
             :proxy_http_basic_authentication=>[proxy_url, "user", "pass"]) {|f|
           assert_equal("200", f.status[0])
           assert_equal("proxy", f.read)
         }
-        assert_match(/#{Regexp.quote url}/, log); log.clear
+        assert_match(/#{Regexp.quote url}/, proxy_auth_log); proxy_auth_log.clear
         assert_raise(ArgumentError) {
-          open("#{url}/proxy",
+          URI.open("#{url}/proxy",
               :proxy_http_basic_authentication=>[true, "user", "pass"]) {}
         }
-        assert_equal("", log); log.clear
+        assert_equal("", proxy_auth_log); proxy_auth_log.clear
       ensure
         proxy.shutdown
+        th.join
       end
+      assert_equal("", proxy_log.string)
+    }
+  end
+
+  def test_authenticated_proxy_http_basic_authentication_success
+    with_http {|srv, dr, url|
+      proxy_log = StringIO.new(''.dup)
+      proxy_logger = WEBrick::Log.new(proxy_log, WEBrick::BasicLog::WARN)
+      proxy_auth_log = ''.dup
+      proxy = WEBrick::HTTPProxyServer.new({
+        :ServerType => Thread,
+        :Logger => proxy_logger,
+        :AccessLog => [[NullLog, ""]],
+        :ProxyAuthProc => lambda {|req, res|
+          proxy_auth_log << req.request_line
+          if req["Proxy-Authorization"] != "Basic #{['user:pass'].pack('m').chomp}"
+            raise WEBrick::HTTPStatus::ProxyAuthenticationRequired
+          end
+        },
+        :BindAddress => '127.0.0.1',
+        :Port => 0})
+      _, proxy_port, _, proxy_host = proxy.listeners[0].addr
+      proxy_url = "http://user:pass@#{proxy_host}:#{proxy_port}/"
+      begin
+        th = proxy.start
+        srv.mount_proc("/proxy", lambda { |req, res| res.body = "proxy" } )
+        URI.open("#{url}/proxy", :proxy => proxy_url) {|f|
+          assert_equal("200", f.status[0])
+          assert_equal("proxy", f.read)
+        }
+        assert_match(/#{Regexp.quote url}/, proxy_auth_log); proxy_auth_log.clear
+        assert_equal("", proxy_auth_log); proxy_auth_log.clear
+      ensure
+        proxy.shutdown
+        th.join
+      end
+      assert_equal("", proxy_log.string)
     }
   end
 
@@ -282,12 +435,12 @@ class TestOpenURI < Test::Unit::TestCase
       srv.mount_proc("/r1/") {|req, res| res.status = 301; res["location"] = "#{url}/r2"; res.body = "r1" }
       srv.mount_proc("/r2/") {|req, res| res.body = "r2" }
       srv.mount_proc("/to-file/") {|req, res| res.status = 301; res["location"] = "file:///foo" }
-      open("#{url}/r1/") {|f|
+      URI.open("#{url}/r1/") {|f|
         assert_equal("#{url}/r2", f.base_uri.to_s)
         assert_equal("r2", f.read)
       }
-      assert_raise(OpenURI::HTTPRedirect) { open("#{url}/r1/", :redirect=>false) {} }
-      assert_raise(RuntimeError) { open("#{url}/to-file/") {} }
+      assert_raise(OpenURI::HTTPRedirect) { URI.open("#{url}/r1/", :redirect=>false) {} }
+      assert_raise(RuntimeError) { URI.open("#{url}/to-file/") {} }
     }
   end
 
@@ -295,7 +448,7 @@ class TestOpenURI < Test::Unit::TestCase
     with_http {|srv, dr, url|
       srv.mount_proc("/r1/") {|req, res| res.status = 301; res["location"] = "#{url}/r2"; res.body = "r1" }
       srv.mount_proc("/r2/") {|req, res| res.status = 301; res["location"] = "#{url}/r1"; res.body = "r2" }
-      assert_raise(RuntimeError) { open("#{url}/r1/") {} }
+      assert_raise(RuntimeError) { URI.open("#{url}/r1/") {} }
     }
   end
 
@@ -358,29 +511,54 @@ class TestOpenURI < Test::Unit::TestCase
     }
   end
 
-  def test_redirect_auth
+  def setup_redirect_auth(srv, url)
+    srv.mount_proc("/r1/") {|req, res|
+      res.status = 301
+      res["location"] = "#{url}/r2"
+    }
+    srv.mount_proc("/r2/") {|req, res|
+      if req["Authorization"] != "Basic #{['user:pass'].pack('m').chomp}"
+        raise WEBrick::HTTPStatus::Unauthorized
+      end
+      res.body = "r2"
+    }
+  end
+
+  def test_redirect_auth_success
     with_http {|srv, dr, url|
-      srv.mount_proc("/r1/") {|req, res| res.status = 301; res["location"] = "#{url}/r2" }
-      srv.mount_proc("/r2/") {|req, res|
-        if req["Authorization"] != "Basic #{['user:pass'].pack('m').chomp}"
-          raise WEBrick::HTTPStatus::Unauthorized
-        end
-        res.body = "r2"
-      }
-      exc = assert_raise(OpenURI::HTTPError) { open("#{url}/r2/") {} }
-      assert_equal("401", exc.io.status[0])
-      open("#{url}/r2/", :http_basic_authentication=>['user', 'pass']) {|f|
+      setup_redirect_auth(srv, url)
+      URI.open("#{url}/r2/", :http_basic_authentication=>['user', 'pass']) {|f|
         assert_equal("r2", f.read)
       }
-      exc = assert_raise(OpenURI::HTTPError) { open("#{url}/r1/", :http_basic_authentication=>['user', 'pass']) {} }
+    }
+  end
+
+  def test_redirect_auth_failure_r2
+    log_tester = lambda {|server_log|
+      assert_equal(1, server_log.length)
+      assert_match(/ERROR WEBrick::HTTPStatus::Unauthorized/, server_log[0])
+    }
+    with_http(log_tester) {|srv, dr, url, server_thread, server_log|
+      setup_redirect_auth(srv, url)
+      exc = assert_raise(OpenURI::HTTPError) { URI.open("#{url}/r2/") {} }
+      assert_equal("401", exc.io.status[0])
+    }
+  end
+
+  def test_redirect_auth_failure_r1
+    log_tester = lambda {|server_log|
+      assert_equal(1, server_log.length)
+      assert_match(/ERROR WEBrick::HTTPStatus::Unauthorized/, server_log[0])
+    }
+    with_http(log_tester) {|srv, dr, url, server_thread, server_log|
+      setup_redirect_auth(srv, url)
+      exc = assert_raise(OpenURI::HTTPError) { URI.open("#{url}/r1/", :http_basic_authentication=>['user', 'pass']) {} }
       assert_equal("401", exc.io.status[0])
     }
   end
 
   def test_userinfo
-    if "1.9.0" <= RUBY_VERSION
-      assert_raise(ArgumentError) { open("http://user:pass@127.0.0.1/") {} }
-    end
+    assert_raise(ArgumentError) { URI.open("http://user:pass@127.0.0.1/") {} }
   end
 
   def test_progress
@@ -389,7 +567,7 @@ class TestOpenURI < Test::Unit::TestCase
       srv.mount_proc("/data/") {|req, res| res.body = content }
       length = []
       progress = []
-      open("#{url}/data/",
+      URI.open("#{url}/data/",
            :content_length_proc => lambda {|n| length << n },
            :progress_proc => lambda {|n| progress << n }
           ) {|f|
@@ -409,7 +587,7 @@ class TestOpenURI < Test::Unit::TestCase
       srv.mount_proc("/data/") {|req, res| res.body = content; res.chunked = true }
       length = []
       progress = []
-      open("#{url}/data/",
+      URI.open("#{url}/data/",
            :content_length_proc => lambda {|n| length << n },
            :progress_proc => lambda {|n| progress << n }
           ) {|f|
@@ -435,24 +613,42 @@ class TestOpenURI < Test::Unit::TestCase
   def test_encoding
     with_http {|srv, dr, url|
       content_u8 = "\u3042"
-      content_ej = "\xa2\xa4".force_encoding("euc-jp")
+      content_ej = "\xa2\xa4".dup.force_encoding("euc-jp")
       srv.mount_proc("/u8/") {|req, res| res.body = content_u8; res['content-type'] = 'text/plain; charset=utf-8' }
       srv.mount_proc("/ej/") {|req, res| res.body = content_ej; res['content-type'] = 'TEXT/PLAIN; charset=EUC-JP' }
       srv.mount_proc("/nc/") {|req, res| res.body = "aa"; res['content-type'] = 'Text/Plain' }
-      open("#{url}/u8/") {|f|
+      URI.open("#{url}/u8/") {|f|
         assert_equal(content_u8, f.read)
         assert_equal("text/plain", f.content_type)
         assert_equal("utf-8", f.charset)
       }
-      open("#{url}/ej/") {|f|
+      URI.open("#{url}/ej/") {|f|
         assert_equal(content_ej, f.read)
         assert_equal("text/plain", f.content_type)
         assert_equal("euc-jp", f.charset)
+        assert_equal(Encoding::EUC_JP, f.read.encoding)
       }
-      open("#{url}/nc/") {|f|
+      URI.open("#{url}/ej/", 'r:utf-8') {|f|
+        # override charset with encoding option
+        assert_equal(content_ej.dup.force_encoding('utf-8'), f.read)
+        assert_equal("text/plain", f.content_type)
+        assert_equal("euc-jp", f.charset)
+        assert_equal(Encoding::UTF_8, f.read.encoding)
+      }
+      URI.open("#{url}/ej/", :encoding=>'utf-8') {|f|
+        # override charset with encoding option
+        assert_equal(content_ej.dup.force_encoding('utf-8'), f.read)
+        assert_equal("text/plain", f.content_type)
+        assert_equal("euc-jp", f.charset)
+        assert_equal(Encoding::UTF_8, f.read.encoding)
+      }
+      assert_raise(ArgumentError) {
+        URI.open("#{url}/ej/", 'r:utf-8', :encoding=>'utf-8') {|f| }
+      }
+      URI.open("#{url}/nc/") {|f|
         assert_equal("aa", f.read)
         assert_equal("text/plain", f.content_type)
-        assert_equal("iso-8859-1", f.charset)
+        assert_equal("utf-8", f.charset)
         assert_equal("unknown", f.charset { "unknown" })
       }
     }
@@ -462,7 +658,7 @@ class TestOpenURI < Test::Unit::TestCase
     with_http {|srv, dr, url|
       content_u8 = "\u3042"
       srv.mount_proc("/qu8/") {|req, res| res.body = content_u8; res['content-type'] = 'text/plain; charset="utf\-8"' }
-      open("#{url}/qu8/") {|f|
+      URI.open("#{url}/qu8/") {|f|
         assert_equal(content_u8, f.read)
         assert_equal("text/plain", f.content_type)
         assert_equal("utf-8", f.charset)
@@ -473,7 +669,7 @@ class TestOpenURI < Test::Unit::TestCase
   def test_last_modified
     with_http {|srv, dr, url|
       srv.mount_proc("/data/") {|req, res| res.body = "foo"; res['last-modified'] = 'Fri, 07 Aug 2009 06:05:04 GMT' }
-      open("#{url}/data/") {|f|
+      URI.open("#{url}/data/") {|f|
         assert_equal("foo", f.read)
         assert_equal(Time.utc(2009,8,7,6,5,4), f.last_modified)
       }
@@ -483,19 +679,19 @@ class TestOpenURI < Test::Unit::TestCase
   def test_content_encoding
     with_http {|srv, dr, url|
       content = "abc" * 10000
-      Zlib::GzipWriter.wrap(StringIO.new(content_gz="".force_encoding("ascii-8bit"))) {|z| z.write content }
+      Zlib::GzipWriter.wrap(StringIO.new(content_gz="".b)) {|z| z.write content }
       srv.mount_proc("/data/") {|req, res| res.body = content_gz; res['content-encoding'] = 'gzip' }
       srv.mount_proc("/data2/") {|req, res| res.body = content_gz; res['content-encoding'] = 'gzip'; res.chunked = true }
       srv.mount_proc("/noce/") {|req, res| res.body = content_gz }
-      open("#{url}/data/") {|f|
+      URI.open("#{url}/data/") {|f|
         assert_equal [], f.content_encoding
         assert_equal(content, f.read)
       }
-      open("#{url}/data2/") {|f|
+      URI.open("#{url}/data2/") {|f|
         assert_equal [], f.content_encoding
         assert_equal(content, f.read)
       }
-      open("#{url}/noce/") {|f|
+      URI.open("#{url}/noce/") {|f|
         assert_equal [], f.content_encoding
         assert_equal(content_gz, f.read.force_encoding("ascii-8bit"))
       }
@@ -509,7 +705,7 @@ class TestOpenURI < Test::Unit::TestCase
         res.cookies << "name2=value2; blabla"
         res.body = "foo"
       }
-      open("#{url}/mcookie/") {|f|
+      URI.open("#{url}/mcookie/") {|f|
         assert_equal("foo", f.read)
         assert_equal(["name1=value1; blabla", "name2=value2; blabla"],
                      f.metas['set-cookie'].sort)
@@ -525,7 +721,7 @@ class TestOpenURI < Test::Unit::TestCase
     assert_raise(ArgumentError) { URI("ftp://127.0.0.1/a%0Ab").read }
     assert_raise(ArgumentError) { URI("ftp://127.0.0.1/a%0Db/f").read }
     assert_raise(ArgumentError) { URI("ftp://127.0.0.1/a%0Ab/f").read }
-    assert_raise(URI::InvalidComponentError) { URI("ftp://127.0.0.1/d/f;type=x") }
+    assert_nothing_raised(URI::InvalidComponentError) { URI("ftp://127.0.0.1/d/f;type=x") }
   end
 
   def test_ftp
